#!/usr/bin/python
import numpy as np
import math
import matplotlib
matplotlib.use("qt4agg")
from matplotlib import pyplot as plt
import sys
import time
import subprocess
import random
import copy
import mldiel

optfilename = "SMAwindow_2layer_run2.txt"

makefigure = 0
expotempprofile = 0
lintempprofile = 1
expoparamprofile = 1
linparamprofile = 0
verbose = 0

#This code performs a simulated annealing search for the optimum CL over a set of parameters. 

random.seed()

#optimization variables
temp = 3 #dB - initial temperature in terms of conversion loss. Gives 1/e point for acceptance
totalsteps = 10000
resetfreq = 1000000 #after this amny sucessful steps the previous maximum is found

#expo temp profile
if expotempprofile == 1:
	finaltempfrac = 0.01
	holdatzero = 1000 #number of steps to hold at zero temp
	#fractional decrease in temperature during each step
	if totalsteps > holdatzero:
		tempdecrease = finaltempfrac**(1/(totalsteps-holdatzero))
	else:
		print("Error: Held at zero longer than total number of steps")
if lintempprofile:
	holdathigh = 0.1 #frac of time at initial temp
	holdatlow = 0.02 #frac of time at zero temp
	tempdecrease = temp/(totalsteps*(1-holdathigh-holdatlow))
	
deltadecreaseperthou = 0.75 #determines how delta shrinks with time
deltadecrease = deltadecreaseperthou**(0.001)

#Fixed sim parameters
air_eps = 1
si_eps = 11.66
si_cond = 0.01

#band to average over
freqs = np.linspace(200,400,101)

#swept sim parameters
art1_ff = 0.25
art1_thick = 100
art2_ff = 0.75
art2_thick = 100
si_thick = 3500

#build first param point array
new_paramlist = []
new_paramlist.append(art1_ff)
new_paramlist.append(art1_thick)
new_paramlist.append(art2_ff)
new_paramlist.append(art2_thick)
new_paramlist.append(si_thick)

#parameter ranges
art1_ff_range = [0.01,0.99]
art1_thick_range = [1,400]
art2_ff_range = [0.01,0.99]
art2_thick_range = [1,400]
si_thick_range = [3000,5000]
ranges = []
ranges.append(art1_ff_range)
ranges.append(art1_thick_range)
ranges.append(art2_ff_range)
ranges.append(art2_thick_range)
ranges.append(si_thick_range)

#characteristic widths in parameters
delta_art1_ff = 0.05
delta_art1_thick = 40
delta_art2_ff = 0.05
delta_art2_thick = 40
delta_si_thick = 120
deltalist = []
deltalist.append(delta_art1_ff)
deltalist.append(delta_art1_thick)
deltalist.append(delta_art2_ff)
deltalist.append(delta_art2_thick)
deltalist.append(delta_si_thick)

#initialize arrays for recording sucesses
paramarray = []
paramarray.append(new_paramlist[:])
optimizearray = []
stepnumarray = []

#initialize arrays for recording all calculated
recparamarray =[]
recparamarray.append(new_paramlist[:])
recoptimizearray = []
recstepnumarray = []

print("Solving Initial model")

#make initial dielectric stack up
airlayer = ['air',1,1,-1,0,0]
art1diellayer_parallel = ['artdiel1',si_eps*art1_ff + air_eps*(1-art1_ff),1,art1_thick*1e-6,0,art1_ff*si_cond]
art2diellayer_parallel = ['artdiel2',si_eps*art2_ff + air_eps*(1-art2_ff),1,art2_thick*1e-6,0,art2_ff*si_cond]
art1diellayer_perp = ['artdiel1',si_eps*air_eps/(si_eps*(1-art1_ff)+air_eps*art1_ff),1,art1_thick*1e-6,0,art1_ff*si_cond]
art2diellayer_perp = ['artdiel2',si_eps*air_eps/(si_eps*(1-art2_ff)+air_eps*art2_ff),1,art2_thick*1e-6,0,art2_ff*si_cond]
siliconlayer = ['silicon',si_eps,1,si_thick*1e-6,0,si_cond]

dielstack_parallel = []
dielstack_perp = []
	
dielstack_parallel.append(airlayer)
dielstack_parallel.append(art1diellayer_parallel)
dielstack_parallel.append(art2diellayer_parallel)
dielstack_parallel.append(siliconlayer)
dielstack_parallel.append(art2diellayer_parallel)
dielstack_parallel.append(art1diellayer_parallel)
dielstack_parallel.append(airlayer)
dielstack_parallel = np.array(dielstack_parallel)

dielstack_perp.append(airlayer)
dielstack_perp.append(art1diellayer_perp)
dielstack_perp.append(art2diellayer_perp)
dielstack_perp.append(siliconlayer)
dielstack_perp.append(art2diellayer_perp)
dielstack_perp.append(art1diellayer_perp)
dielstack_perp.append(airlayer)
dielstack_perp = np.array(dielstack_perp)
	
print("Solving Multilayer Dielectric model for:")
print(new_paramlist)
	
#solve model for this point
bandaves_parallel = mldiel.calcBandAve(dielstack_parallel,freqs)
bandaves_perp = mldiel.calcBandAve(dielstack_perp,freqs)
	
print("Multilayer Dielectric model solved")

optimizeparam = -20*math.log10(0.5*(bandaves_parallel[0][0]+bandaves_perp[0][0]))

optimizearray.append(optimizeparam)
recoptimizearray.append(optimizeparam)
stepnumarray.append(0)
recstepnumarray.append(0)
prevoptimizeparam = optimizeparam

print("Starting Optimization")
print(paramarray)
print(optimizearray)

stepnum = 1
while stepnum <= totalsteps:
	
	if verbose:
		print("AT STEP # " + str(stepnum))
	elif stepnum%100 == 0:
		print("AT STEP # " + str(stepnum))

	#now find new point off of last vector in paramarray
	new_paramlist_ref = paramarray[len(paramarray)-1]
	new_paramlist = new_paramlist_ref[:]
	#print("Starting from this point:")
	#print(new_paramlist)
	inrange = 0
	while inrange == 0:
		paramupdate = []
		for i in range(0,len(new_paramlist)):
			paramupdate.append(random.gauss(new_paramlist[i],deltalist[i]))
		inrange = 1
		for i in range(0,len(new_paramlist)):	
			if paramupdate[i] < ranges[i][0] or paramupdate[i] > ranges[i][1]:
				inrange = 0
		if inrange == 1:
			new_paramlist = paramupdate[:]
	#print("Trying this point:")
	#print(new_paramlist)
	

	#make diel stack up

	#make new dielectric stack up
	airlayer = ['air',1,1,-1,0,0]
	art1diellayer_parallel = ['artdiel1',si_eps*new_paramlist[0] + air_eps*(1-new_paramlist[0]),1,new_paramlist[1]*1e-6,0,new_paramlist[0]*si_cond]
	art2diellayer_parallel = ['artdiel2',si_eps*new_paramlist[2] + air_eps*(1-new_paramlist[2]),1,new_paramlist[3]*1e-6,0,new_paramlist[2]*si_cond]
	art1diellayer_perp = ['artdiel1',si_eps*air_eps/(si_eps*(1-new_paramlist[0])+air_eps*new_paramlist[0]),1,new_paramlist[1]*1e-6,0,new_paramlist[0]*si_cond]
	art2diellayer_perp = ['artdiel2',si_eps*air_eps/(si_eps*(1-new_paramlist[2])+air_eps*new_paramlist[2]),1,new_paramlist[3]*1e-6,0,new_paramlist[2]*si_cond]
	siliconlayer = ['silicon',si_eps,1,new_paramlist[4]*1e-6,0,si_cond]

	dielstack_parallel = []
	dielstack_perp = []
		
	dielstack_parallel.append(airlayer)
	dielstack_parallel.append(art1diellayer_parallel)
	dielstack_parallel.append(art2diellayer_parallel)
	dielstack_parallel.append(siliconlayer)
	dielstack_parallel.append(art2diellayer_parallel)
	dielstack_parallel.append(art1diellayer_parallel)
	dielstack_parallel.append(airlayer)
	dielstack_parallel = np.array(dielstack_parallel)

	dielstack_perp.append(airlayer)
	dielstack_perp.append(art1diellayer_perp)
	dielstack_perp.append(art2diellayer_perp)
	dielstack_perp.append(siliconlayer)
	dielstack_perp.append(art2diellayer_perp)
	dielstack_perp.append(art1diellayer_perp)
	dielstack_perp.append(airlayer)
	dielstack_perp = np.array(dielstack_perp)
		
	if verbose:
		print("Solving Multilayer Dielectric model for:")
		print(new_paramlist)
		
	#solve model for this point
	bandaves_parallel = mldiel.calcBandAve(dielstack_parallel,freqs)
	bandaves_perp = mldiel.calcBandAve(dielstack_perp,freqs)
		
	if verbose:
		print("Multilayer Model solved")

	optimizeparam = -20*math.log10(0.5*np.sqrt(bandaves_parallel[0][0]*bandaves_parallel[0][0]+bandaves_perp[0][0]*bandaves_perp[0][0]))
	
	#decide if to keep this point or choose another
	delta_opt = optimizeparam - prevoptimizeparam
	if delta_opt > 0  and temp <= 0 :
		if verbose:
			print("The new model is better and accepted at zero temp: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
		paramarray.append(new_paramlist[:])
		prevoptimizeparam = optimizeparam
		optimizearray.append(optimizeparam)
		stepnumarray.append(stepnum)
	elif temp <= 0 and delta_opt <= 0:
		if verbose:
			print("The new model is worse and rejeected at zero temp")
	elif delta_opt > 0 or random.random() < math.exp(delta_opt/temp):
		if delta_opt > 0:
			if verbose:
				print("The new model is better and accepted: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
		else:
			if verbose:
				print("The new model is worse but accepted at temp " + str(temp) +  "dB : " + str(optimizeparam) + " < " + str(prevoptimizeparam))
		paramarray.append(new_paramlist[:])
		prevoptimizeparam = optimizeparam
		optimizearray.append(optimizeparam)
		stepnumarray.append(stepnum)
	else:
		if verbose:
			print("The new model is rejected: " + str(optimizeparam) + " < " + str(prevoptimizeparam))
	
	#now record all steps for plotting
	recparamarray.append(new_paramlist[:])
	recoptimizearray.append(optimizeparam)
	recstepnumarray.append(stepnum)
		
	stepnum = stepnum + 1
	if expotempprofile == 1:
		if stepnum <= totalsteps - holdatzero:
			temp = temp*tempdecrease
		else:
			temp = 0
	if lintempprofile == 1:
		if stepnum > holdatlow*totalsteps:
			temp = temp - tempdecrease
			if temp < 0:
				temp = 0

print("Writing output file to: " + optfilename)

fout = open(optfilename,"w")
for linenum in range(0,len(recoptimizearray)):
	fout.write(str(recparamarray[linenum][0]) + " " + str(recparamarray[linenum][1]) + " " + str(recparamarray[linenum][2]) + " " + str(recparamarray[linenum][3]) + " " + str(recparamarray[linenum][4]) + " " + str(recoptimizearray[linenum]) + "\n")	
fout.close()