#!/usr/bin/python
import numpy as np
import math
import cmath
import matplotlib
from matplotlib import pyplot as plt
import sys
import time
import mldiel

air_eps = 1
art1_ff = 0.18
art1_thick = 150
art2_ff = 0.75
art2_thick = 126
si_thick = 3183
si_cond = 0.01 # 10000ohm*cm
si_eps = 11.66

#make dielectric stack: name, epsilon_real, mu_real, thickness (m), dielectric loss tangent, conductivity (S / m)
airlayer = ['air',1,1,-1,0,0]
art1diellayer_parallel = ['artdiel1',si_eps*art1_ff + air_eps*(1-art1_ff),1,art1_thick*1e-6,0,art1_ff*si_cond]
art2diellayer_parallel = ['artdiel2',si_eps*art2_ff + air_eps*(1-art2_ff),1,art2_thick*1e-6,0,art2_ff*si_cond]
art1diellayer_perp = ['artdiel1',si_eps*air_eps/(si_eps*(1-art1_ff)+air_eps*art1_ff),1,art1_thick*1e-6,0,art1_ff*si_cond]
art2diellayer_perp = ['artdiel2',si_eps*air_eps/(si_eps*(1-art2_ff)+air_eps*art2_ff),1,art2_thick*1e-6,0,art2_ff*si_cond]
siliconlayer = ['silicon',si_eps,1,si_thick*1e-6,0,si_cond]

dielstack_parallel = []
dielstack_perp = []
	
dielstack_parallel.append(airlayer)
dielstack_parallel.append(art1diellayer_parallel)
dielstack_parallel.append(art2diellayer_parallel)
dielstack_parallel.append(siliconlayer)
dielstack_parallel.append(art2diellayer_parallel)
dielstack_parallel.append(art1diellayer_parallel)
dielstack_parallel.append(airlayer)
dielstack_parallel = np.array(dielstack_parallel)

dielstack_perp.append(airlayer)
dielstack_perp.append(art1diellayer_perp)
dielstack_perp.append(art2diellayer_perp)
dielstack_perp.append(siliconlayer)
dielstack_perp.append(art2diellayer_perp)
dielstack_perp.append(art1diellayer_perp)
dielstack_perp.append(airlayer)
dielstack_perp = np.array(dielstack_perp)
	
freqs = np.linspace(1,1000,10000)
s11_parallel = []
s12_parallel = []
s21_parallel = []
s22_parallel = []

s11_perp = []
s12_perp = []
s21_perp = []
s22_perp = []

for freq in freqs:	
	smatrix = mldiel.calcSparams(dielstack_parallel,freq*1e9)
	s11_parallel.append(20*math.log10(np.absolute(smatrix[0][0])))
	s12_parallel.append(20*math.log10(np.absolute(smatrix[0][1])))
	s21_parallel.append(20*math.log10(np.absolute(smatrix[1][0])))
	s22_parallel.append(20*math.log10(np.absolute(smatrix[1][1])))
	smatrix = mldiel.calcSparams(dielstack_perp,freq*1e9)
	s11_perp.append(20*math.log10(np.absolute(smatrix[0][0])))
	s12_perp.append(20*math.log10(np.absolute(smatrix[0][1])))
	s21_perp.append(20*math.log10(np.absolute(smatrix[1][0])))
	s22_perp.append(20*math.log10(np.absolute(smatrix[1][1])))
	
s11_parallel = np.array(s11_parallel)
s12_parallel = np.array(s12_parallel)
s21_parallel = np.array(s21_parallel)
s22_parallel = np.array(s22_parallel)

s11_perp = np.array(s11_perp)
s12_perp = np.array(s12_perp)
s21_perp = np.array(s21_perp)
s22_perp = np.array(s22_perp)

fig1 = plt.figure(1)
ax11 = fig1.add_subplot(221)
ax12 = fig1.add_subplot(222)
ax13 = fig1.add_subplot(223)
ax14 = fig1.add_subplot(224)

ax11.plot(freqs,s11_parallel,color='k')
ax12.plot(freqs,s12_parallel,color='k')
ax13.plot(freqs,s21_parallel,color='k')
ax14.plot(freqs,s22_parallel,color='k')

ax11.plot(freqs,s11_perp,color='r')
ax12.plot(freqs,s12_perp,color='r')
ax13.plot(freqs,s21_perp,color='r')
ax14.plot(freqs,s22_perp,color='r')

plt.show()	